
/*
Do Filtered Statistics update as frequently as normal statistics?            
I received a question this week about whether filtered statistics update as frequently as regular statistics.  The right way to ask the question is Do filtered statistics become invalid as frequently as regular statistics?, as stats are recomputed based on the queries that consume them, while stats are invalidated based on changes to the underlying data.
The basic mechanism for stats invalidation is a per-column counter in the storage engine.  Its not transacted, and it is incremented on each INSERT/UPDATE/DELETE/MERGE.  When a normal statistics object is created, the current modcounter for that column is stored in the statistics object.  As changes are made to the table, the difference between the current modcounter and the one stored in the statistics object becomes greater.  Ultimately, when the difference is large enough (I will slightly oversimplify and say that it is 20% of the table changed for this blog post), the statistics object is invalidated.  When a query consuming the statistic is executed again, it will determine that the statistic is out-of-date and recompile it.  Once done, the query is recompiled using the new statistics object.  Unfortunately, many of the details of this logic are not exposed, and this can make it somewhat difficult to understand the mechanism.
 
Filtered statistics do not change the fundamental mechanism  there is still one set of counters in the storage engine.  The thresholds for when filtered statistics become invalid, however, are modified from the normal case based on the estimated selectivity of the filter.  So, a filter that only covers 20% of the domain is updated 1/5th as frequently.  The calculation is done without knowledge to whether the changes being made to the table would actually cause the statistic object to be changed.
Ive written up an example to show that filtered stats do indeed get updated even when all of the changes to the data are outside of the range of the filter on the statistics object.  In this example, the filter selectivity is very high (~90%), and the data change is effectively not really changing the data, but I know that it is impacting the modcounter for the column.
*/

use t1
go

--drop table testfilteredstatsrefresh
-- create a table for our example with values 0...999 in col2
create table testfilteredstatsrefresh(col1 int identity, col2 int)
go

set nocount on
begin transaction
declare @a int=0
while @a < 20000
begin
insert into testfilteredstatsrefresh(col2) values (@a % 1000)
set @a+=1
end
commit transaction
go

-- ~80% of the table
select COUNT(*) from testfilteredstatsrefresh WHERE col2 > 100
-- ~20% of the table
select COUNT(*) from testfilteredstatsrefresh WHERE col2 > 900
go

-- create filtered stats over a portion of the domain
create statistics i1 on testfilteredstatsrefresh(col2) WHERE col2 > 100 
go
create statistics i2 on testfilteredstatsrefresh(col2) WHERE col2 > 900 
go

dbcc show_statistics (testfilteredstatsrefresh, i1)
dbcc show_statistics (testfilteredstatsrefresh, i2)

select * from testfilteredstatsrefresh where col2 <> 55 option(recompile)
go
set nocount off
-- update the table with a series of no-ops on the section of the range
-- not covered by the filtered stats
update testfilteredstatsrefresh set col2=col2 %1000 where col2<100
update testfilteredstatsrefresh set col2=col2 %1000 where col2<100
update testfilteredstatsrefresh set col2=col2 %1000 where col2<100
update testfilteredstatsrefresh set col2=col2 %1000 where col2<100
update testfilteredstatsrefresh set col2=col2 %1000 where col2<100
go

-- run the query again - auto-stats fires again
-- depending on which query you run different stats will auto-update
-- so, 
select * from testfilteredstatsrefresh where col2 <> 55 option(recompile)

select * from testfilteredstatsrefresh where col2> 100 option(recompile)

select * from testfilteredstatsrefresh where col2> 400 option(recompile)

select * from testfilteredstatsrefresh where col2> 900 option(recompile)

select STATS_DATE(object_id, stats_id) AS statistics_update_date, *
FROM sys.stats 
WHERE object_id = OBJECT_ID('dbo.testfilteredstatsrefresh');

-- drop table and start over
drop table testfilteredstatsrefresh
