
/*
I thought that it would be good to play with disjoint ranges and see whether they can be "covered" 
by a filtered index.  In my last blog post, I covered how a basic, single range case *is* 
covered by a filtered index when the predicates match up correctly.  In this post, I'll 
try multiple ranges to see what happens.

Same database setup as before (just a table with wide rows and enough rows to make index 
choice obvious to the optimizer):
*/

create database t1
go
use t1
go

drop table t1
go

create table t1(col1 int,
col2 nchar(2000),
col3 time)
go

create index i1 on t1(col1) where col1 > 5 and col1 < 20
go


declare @p int = 0
while @p < 20000
begin
  insert into t1(col1) values (rand()*10000)
  set @p=@p+1
end
go

create index i2 on t1(col1) where col1 > 25 and col1 < 40
go

/*
OK I added a second index over a different range in the same column. This is not a
case that the SQL Server 2005 optimizer had to handle.
*/

-- query 1: a query with a disjunction (OR) of ranges (BETWEEN/AND).
select col1 from t1 
where (col1 > 5 and col1 < 20) or (col1 > 25 and col1 < 40)

/*
|--Table Scan(OBJECT:([t1].[dbo].[t1]), WHERE:([t1].[dbo].[t1].[col1]>(5) AND [t1].[dbo].[t1].[col1]<(20)
OR [t1].[dbo].[t1].[col1]>(25) AND [t1].[dbo].[t1].[col1]<(40)))

Hrm. Well, there doesn't seem to be code that does index unions for cases like
this, unfortunately.

Hinting isn't an option either because the index hint code returns an error if the
hinted index does not completely cover the index. On my machine, I get: 

(43 row(s) affected)
SQL Server Execution Times:
CPU time = 47 ms, elapsed time = 2661 ms.

That elapsed time is mostly reading pages off my IDE drive. To be clear, this is the
_cold cache_ time, which means that I ran "DBCC dropcleanbuffers" before my run. This throws out all
of the buffer pool pages and forces a read from disk. One assumption in the costing model for the SQL
Server QP is that the pages are _not_ in memory already (and there are cases where this becomes interesting).

For this example, it just means that we have a big table scan with lots of IOs to do (and is therefore
slow).
*/


-- Second example: IN lists
select col1 from t1 where col1 in (6, 7, 8, 26)

/*
|--Table Scan(OBJECT:([t1].[dbo].[t1]), WHERE:([t1].[dbo].[t1].[col1]=(6) OR [t1].[dbo].[t1].[col1]=(7)
OR [t1].[dbo].[t1].[col1]=(8) OR [t1].[dbo].[t1].[col1]=(26)))

Well, this is consistent.  IN is generally just a list of ORs as far as the QP is concerned.  
Runtime for this query should be very similar to what we saw in our last example:

(7 row(s) affected)
 SQL Server Execution Times:
   CPU time = 63 ms,  elapsed time = 2708 ms.

OK, so what's an enterprising SQL developer to do?  Well, you can rewrite the query as a 
UNION ALL as long as you know that the ranges are disjoint and cover the predicate properly.
*/

select col1 from t1 with (index=i1) 
where (col1 > 5 and col1 < 20) 
union all 
select col1 from t1 with (index=i2) where (col1 > 25 and col1 < 40) 

/*
SQL Server parse and compile time: 
   CPU time = 0 ms, elapsed time = 0 ms.
(43 row(s) affected)
 SQL Server Execution Times:
   CPU time = 0 ms,  elapsed time = 0 ms.
*/